function love.load()
    ball = {}
    ball.x = 0
    ball.y = 0
    exit = {}
    exit.x = 0
    exit.y = 0
    tilecount = 0
    time = 0
    tilescollected = 0
    gamestate = "start"
    font1 = love.graphics.newFont(36)
    font2 = love.graphics.newFont(64)
    startmessage = love.graphics.newText(font1, "Press Space to start.")
    gomessage = love.graphics.newText(font2, "Go!")
    deadmessage = love.graphics.newText(font1, "Sorry!")
    levelupmessage = love.graphics.newText(font1, "Well Done!")
    finishedmessage = love.graphics.newText(font1, "Yay! You've finished\nthis little game.")
    level = 1
    files = love.filesystem.getDirectoryItems("maps")
    levelcount = 0
    for _,level in ipairs(files) do
        levelcount = levelcount + 1
    end
    maps = {}
    for _,m in ipairs(files) do table.insert(maps, "maps/"..m) end
    table.sort(maps)
    loadMap(maps[level])
end

function love.update(dt)
    if gamestate == "waiting" then
        time = time + dt
        if time > 3 then
            gamestate = "going"
            time = 0
        end
    end
end

function love.draw()
    if gamestate == "waiting" then
    	drawMap()
    	love.graphics.setColor(255, 120, 0)
	    love.graphics.rectangle("fill", exit.x, exit.y, 32, 32)
        love.graphics.setColor(255, 255, 50)
        love.graphics.circle("fill", ball.x + 16, ball.y + 16, 12)
    elseif gamestate == "dead" then
        love.graphics.setColor(255, 255, 255)
        love.graphics.draw(deadmessage, 400 - deadmessage:getWidth() / 2, 282)
    elseif gamestate == "levelup" then
        love.graphics.setColor(255, 255, 255)
        love.graphics.draw(levelupmessage, 400 - levelupmessage:getWidth() / 2, 282)
    elseif gamestate == "start" then
        love.graphics.setColor(255, 255, 255)
        love.graphics.draw(startmessage, 400 - startmessage:getWidth() / 2, 282)
    elseif gamestate == "going" then
        love.graphics.setColor(255, 255, 255)
        love.graphics.draw(gomessage, 400 - gomessage:getWidth() / 2, 268)
    elseif gamestate == "finished" then
        love.graphics.setColor(255, 255, 255)
        love.graphics.draw(finishedmessage, 400 - finishedmessage:getWidth() / 2, 268)
    end
end

function love.keypressed(key)
    if key == "escape" then
        love.event.quit()
    end
    if key == "space" and gamestate == "start" then
        gamestate = "waiting"
    end
    if key == "space" and gamestate == "levelup" then
        gamestate = "start"
        loadMap(maps[level])
    end
    if key == "space" and gamestate == "dead" then
        gamestate = "start"
        loadMap(maps[level])
    end
    if key == "space" and gamestate == "finished" then
        love.event.quit()
    end
    if key == "right" then
        go(32, 0)
    end
    if key == "left" then
        go(-32, 0)
    end
    if key == "up" then
        go(0, -32)
    end
    if key == "down" then
        go(0, 32)
    end
end

function go(dx, dy)
    if gamestate == "going" then
        local move = true
        ball.x = ball.x + dx
        ball.y = ball.y + dy
        for _,tiletable in ipairs(tilecorners) do
            local x,y = tiletable[1],tiletable[2]
            if ball.x == x and ball.y == y then
                tilescollected = tilescollected + 1 
            end
        end
        for _,blacktable in ipairs(blackcorners) do
            local x,y = blacktable[1],blacktable[2]
            if ball.x == x and ball.y == y then
                gamestate = "dead"
            end
        end
        if ball.x == exit.x and ball.y == exit.y then
            if tilescollected == tilecount then
                gamestate = "levelup"
                level = level + 1
                if level > levelcount then
                    gamestate = "finished"
                end
            end
	    end
    end
end

function loadMap(path)
    tileString = love.filesystem.load(path)() 
	love.graphics.clear()
	tilecorners = {}
	blackcorners = {}
	exit.x,exit.y = 0,0
	ball.x,ball.y = 0,0
	cb = 0
	bc = 0
    tilecount = 0
    tilescollected = 0
    gamestate = "start"

    tileTable = {}

    local width = #(tileString:match("[^\n]+"))

    for x = 1,width,1 do tileTable[x] = {} end

    local rowIndex,columnIndex = 1,1
    for row in tileString:gmatch("[^\n]+") do
    	assert(#row == width, 'Map is not aligned: width of row ' .. tostring(rowIndex) .. ' should be ' .. tostring(width) .. ', but it is ' .. tostring(#row))
    	columnIndex = 1
        for character in row:gmatch(".") do
            tileTable[columnIndex][rowIndex] = character
			if character == " " then
	  			cb = cb + 1
	  			tilecorners[cb] = { (columnIndex - 1)*32 + 32, (rowIndex - 1)*32 + 32, rowIndex, columnIndex }
                tilecount = tilecount + 1
			end
			if character == "#" then
	  			bc = bc + 1
	  			blackcorners[bc] = { (columnIndex - 1)*32 + 32, (rowIndex - 1)*32 + 32, rowIndex, columnIndex }
			end
			if character == "X" then
			    exit.x = (columnIndex - 1)*32 + 32
	  			exit.y = (rowIndex - 1)*32 + 32
			end
    		if character == "P" then
      			ball.x = (columnIndex - 1)*32 + 32
      			ball.y = (rowIndex - 1)*32 + 32
                cb = cb + 1
	  			tilecorners[cb] = { (columnIndex - 1)*32 + 32, (rowIndex - 1)*32 + 32, rowIndex, columnIndex }
    		end
      		columnIndex = columnIndex + 1
    	end
    	rowIndex=rowIndex+1
  	end
end

function drawMap()
    for _,tiletable in ipairs(tilecorners) do
    	local x,y = tiletable[1], tiletable[2]
		love.graphics.setColor(80, 255, 80)
    	love.graphics.rectangle("fill", x, y, 32, 32)
        love.graphics.setColor(0, 0, 0)
        love.graphics.rectangle("line", x, y, 32, 32)
  	end
    for _,blacktable in ipairs(blackcorners) do
    	local x,y = blacktable[1], blacktable[2]
		love.graphics.setColor(0, 0, 0)
    	love.graphics.rectangle("fill", x, y, 32, 32)
  	end
end
